"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

Launcher for nRoBo framework.

@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import time

from nrobo import *
from nrobo.cli import *
from nrobo.cli.cli_constants import *
from nrobo.cli.install import *
from nrobo.cli.install import install_nrobo
from nrobo.cli.nglobals import *

from nrobo.util.process import *
from nrobo.cli.nrobo_args import SHOW_ONLY_SWITCHES
import nrobo.cli.detection as detect

global __REQUIREMENTS__


def launcher_command(exit_on_failure=True):
    """Prepares nrobo launcher command
       by parsing command line switches
       in order to trigger test suite launch.

       Returns [str], args: command list, actual args
    """

    # Need to import set_environment method here
    # to handle circular import of partially initialized module
    from nrobo import set_environment
    import argparse

    set_environment()

    # parse command line arguments
    from nrobo.cli.nrobo_args import nrobo_cli_parser
    args = nrobo_cli_parser(exit_on_failure=exit_on_failure)

    # process each nrobo cli arguments
    if args.install:
        # Install dependencies
        with console.status(f"[{STYLE.TASK}]Installing dependencies...\n"):
            install_nrobo(None)
            return None, None, None
    if args.VERSION:
        # show version
        from nrobo import __version__
        console.print(f"nrobo {__version__}\n")
        return None, None, None
    if args.suppress:
        # suppress upgrade prompt
        os.environ[EnvKeys.SUPPRESS_PROMPT] = '0'
    if args.version:
        result = terminal(['pytest', f"--version"], debug=True, text=True, capture_output=True)
        console.print(f"{result.stdout}")
        return None, None, None

    # build pytest launcher command
    command = ["pytest"]  # start with programme name
    command_builder_notes = []  # list for storing notes during cli switch processing
    override_defaults = []

    # process other switches
    with console.status(f"[{STYLE.TASK}]Parsing command-line-args...\n"):
        for key, value in args.__dict__.items():
            """break each arg into key, value pairs and process each key"""

            # handle hyphens in key names since argparser replaces hyphes with underscore
            # while parsing cli args
            # this, replace hyphen with dash if present_release in key
            key = key.replace('_', '-')

            if value:
                """if key has value, then only proceed with current key"""

                if type(value) is bool or isinstance(value, bool):
                    """if a bool key is found, only add key to the launcher command, not the value
                        and proceed with next key"""
                    if key == nCLI.SUPPRESS or key == nCLI.FULLPAGE_SCREENSHOT:
                        continue
                    elif key in SHOW_ONLY_SWITCHES:
                        terminal(['pytest', f"--{key}"], debug=True)
                        return None, None, None
                    elif key == args.version:
                        terminal(['pytest', f"--{key}"], debug=True)
                        exit()
                    elif key == 'capture-no':
                        command.append('-s')
                        continue
                    elif key == 'extra-summary':
                        command.append('-r')
                        continue
                    else:
                        command.append(f"--{key}")
                        continue
                elif key not in nCLI.ARGS.keys():
                    """process special short keys(single letter keys) that does not have corresponding long key"""
                    if key == "c":
                        "process key==-c"
                        command.append(f"-{key}")
                        command.append(str(value))
                    else:
                        """simply add long keys to launcher command"""
                        if key == nCLI.TARGET:
                            continue  # DO NOT ADD TO PYTEST LAUNCHER
                        if f"--{key}" in nCLI.DEFAULT_ARGS:
                            override_defaults.append(f"--{key}")

                        command.append(f"--{key}")
                        command.append(str(value))
                elif key in nCLI.ARGS:
                    """process nrobo specific keys"""
                    if key in [nCLI.APP, nCLI.URL, nCLI.USERNAME, nCLI.PASSWORD, nCLI.BROWSER_CONFIG]:
                        if key == nCLI.APP:
                            os.environ[EnvKeys.APP] = value
                        elif key == nCLI.URL:
                            os.environ[EnvKeys.URL] = value
                        elif key == nCLI.USERNAME:
                            os.environ[EnvKeys.USERNAME] = value
                        elif key == nCLI.PASSWORD:
                            os.environ[EnvKeys.PASSWORD] = value

                        # add keys to launcher command
                        command.append(f"--{key}")
                        command.append(str(value))
                        continue

                    if key == nCLI.BROWSER:
                        os.environ[EnvKeys.BROWSER] = value
                        raise_exception_if_browser_not_supported(os.environ[EnvKeys.BROWSER])
                        if args.fullpagescreenshot:
                            command.append(f"--{nCLI.BROWSER}")
                            command.append(str(Browsers.CHROME_HEADLESS))
                            if f"--{nCLI.FULLPAGE_SCREENSHOT}" not in command:
                                command.append(f"--{nCLI.FULLPAGE_SCREENSHOT}")
                            command_builder_notes.append(
                                f"[{STYLE.HLOrange}]\tOverride --browser switch value with {Browsers.CHROME_HEADLESS} because fullpagescreenshot switch is enabled...")
                        else:
                            command.append(f"--{key}")
                            command.append(str(value))
                        continue
                    elif key == nCLI.MARKER:
                        command.append(f"-m")
                        command.append(str(value))
                        continue
                    elif key == nCLI.KEY:
                        command.append(f"-k")
                        command.append(value)
                        continue
                    elif key == nCLI.INSTANCES:
                        command.append(f"-n")
                        command.append(str(value))
                    elif key == nCLI.RERUNS:
                        command.append(f"--{key}")
                        command.append(value)
                        continue
                    elif key == nCLI.REPORT:
                        if str(value).lower() not in [NREPORT.HTML, NREPORT.ALLURE]:
                            console.print(f"Incorrect report type! Valid report types are html | allure.")
                            exit(1)
                        if str(value).lower() in NREPORT.HTML:
                            command.append(f"--{NREPORT.HTML}")
                            command.append(f"{Path(NREPORT.REPORT_DIR) / args.target}")
                        elif str(value).lower() in NREPORT.ALLURE:
                            command.append(f"--{NREPORT.HTML}")
                            command.append(f"{Path(NREPORT.REPORT_DIR) / args.target}")
                            # command.append(f"--allure-no-capture")

                            # Doc: https://allurereport.org/docs/gettingstarted-installation/
                    else:
                        if key == nCLI.TARGET:
                            continue  # DO NOT ADD TO PYTEST LAUNCHER
                        command.append(f"--{key}")
                        command.append(value)

    if not args.browser:
        """browser not provided"""

        if args.fullpagescreenshot:
            command.append(f"--{nCLI.BROWSER}")
            command.append(str(Browsers.CHROME_HEADLESS))
            command.append(f"--{nCLI.FULLPAGE_SCREENSHOT}")
            command_builder_notes.append(
                f"[{STYLE.HLOrange}]\t--browser switch was missing. Default browser {Browsers.CHROME_HEADLESS} is "
                f"selected because fullpagescreenshot switch is enabled...")
        else:
            command.append(f"--{nCLI.BROWSER}")
            command.append(f"{Browsers.CHROME}")
            command_builder_notes.append(
                f"[{STYLE.HLOrange}]\t--browser switch was missing. Default browser {Browsers.CHROME} is selected...")
    if not args.rootdir:
        command_builder_notes.append(
            f"[{STYLE.HLOrange}]\t--rootdir switch was missing. Default test path <current-dir> is selected...")

    if not args.alluredir:
        command.append(f"--alluredir")
        command.append(f"{NREPORT.ALLURE_REPORT_PATH}")

    if not args.pythonwarnings and args.browser == Browsers.ANTI_BOT_CHROME:
        # Suppress DeprecationWarning when running in undetected_chromedriver
        # Due to the following error:
        #  /Users/einsteinpanchdev/webdev/nrobo/venv/lib/python3.12/site-packages/undetected_chromedriver/__init__.py:363:
        #  DeprecationWarning: 'locale.getdefaultlocale' is deprecated and slated for removal in Python 3.15.
        #  Use setlocale(), getencoding() and getlocale() instead.
        command.append(f"-W")
        command.append(f"ignore::DeprecationWarning")

    # Add single parameter commands by default
    # That make sense.
    # command.append("-V") # This setting is not working. With this, tests are even not running at all.

    for k, v in nCLI.DEFAULT_ARGS.items():
        if k in override_defaults:
            continue  # skip adding k,v pair if it is already added by arg parse
        command = command + v

    return command, args, command_builder_notes


def launch_nrobo():
    """Parse command-line-arguments"""

    command, args, command_builder_notes = launcher_command()

    if command is None and args is None and command_builder_notes is None:
        return

    with console.status(f"[{STYLE.TASK}]:smiley: Running tests. Press Ctrl+C to exit nRoBo.\n"):

        if detect.developer_machine():
            console.print(f"[{STYLE.INFO}]{command}")

        if args.report and args.report == NREPORT.ALLURE:  # test if needed allure report
            create_allure_report(command)
        else:
            create_simple_html_report(command)

        print_notes(command_builder_notes)


def create_allure_report(command: list) -> int:
    """prepares allure report based on pytest launcher <command>"""

    console.print(f"[{STYLE.HLGreen}]Running tests and preparing allure report")

    _ALLURE_DIR = '--alluredir'
    allure_results = (Path(os.environ[EnvKeys.EXEC_DIR]) / "results" / "allure-results")
    if _ALLURE_DIR in command:
        allure_results = command[command.index(_ALLURE_DIR) + 1]
    terminal(command + [_ALLURE_DIR, allure_results], debug=True, use_os_system_call=True)

    if _ALLURE_DIR in command:
        allure_generated_report = Path(allure_results) / "allure-report"
    else:
        allure_generated_report = allure_results.parent / "allure-report"

    console.print(f"[{STYLE.HLGreen}]Preparing allure report")

    terminal([NREPORT.ALLURE, "generate", "--name", "nRoBo TEST REPORT", "-o", allure_generated_report, "--clean",
              allure_results], use_os_system_call=True)

    terminal([NREPORT.ALLURE, "serve", allure_results], use_os_system_call=True)


def create_simple_html_report(command: list) -> int:
    """prepares simple html report based on pytest launcher command"""
    console.print(f"[{STYLE.HLGreen}]Running tests and preparing html report")

    return_code = terminal(command, debug=True, use_os_system_call=True)
    console.rule(
        f"[{STYLE.HLOrange}]Report is ready at file://{Path(os.environ[EnvKeys.EXEC_DIR]) / Path(NREPORT.REPORT_DIR) / NREPORT.HTML_REPORT_NAME}")

    return return_code


def print_notes(notes: list):
    """print notes"""

    if len(notes) == 1:
        console.print(f":warning:[{STYLE.HLOrange}] Note:")
    elif len(notes) >= 2:
        console.print(f":warning:[{STYLE.HLOrange}] Notes:")

    if notes:
        for note in notes:
            console.print(note)



import pytest
from selenium.webdriver.common.by import By

from pages import Page


class TestActionChainMethods:

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_action_chain_reference(self, driver, logger):
        """Example of action chain reference"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkDragAndDrop = (By.CSS_SELECTOR, "[href='/drag_and_drop']")
        page.click(*lnkDragAndDrop)

        boxHeaderA = (By.XPATH, "//header[text()='A']")
        boxHeaderB = (By.XPATH, "//header[text()='B']")

        eleHeaderA = page.find_element(*boxHeaderA)
        eleHeaderB = page.find_element(*boxHeaderB)
        page.action_chain().drag_and_drop(eleHeaderA, eleHeaderB).perform()

        page.wait_for_a_while(4)





"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import logging
from selenium.webdriver.remote.webdriver import WebDriver
from typing import Union
from nrobo.selenese import NRobo

AnyBrowser = Union[None, WebDriver]
from selenium.webdriver.common.actions.wheel_input import WheelInput
from selenium.webdriver.common.actions.pointer_input import PointerInput
from selenium.webdriver.common.actions.key_input import KeyInput

AnyDevice = Union[PointerInput, KeyInput, WheelInput]


class Page(NRobo):
    """
    Page class is the base class for every _page objects
    that one is going to be created for his/her project.

    Page class inherits wrapper classes wraps the most
    frequently used api's of the Webdriver, WebElement,
    and other selenium classes which brings the power of
    readability of your scripts. Along with readable code,
    comes the power of highly maintainable and understandable
    code-base.

    And many more...
    I would request to checkout video tutorials uploaded at the
    following YouTube channel: https://shorturl.at/lpqKS

    Example usage: Assume that you want to create a Page Object
    in your automation project for home _page, then you should
    declare your Page Class as following:

    Package: pages
    File: home.py
    ============FileContent of home.py===========================

    from pages import Page

    class PageHome(Page):

        def __init__(self, driver, logger):
            super().__init__(driver, logger)

            # Definition of home _page locators should go below

        def page_method_1(self):
            ...

        def page_mathod_2(self):
            ...

        # and so on per project need.
    """

    def __init__(self, driver=AnyBrowser, logger=None | logging.Logger, duration: int = 250,
                 devices: list[AnyDevice] | None = None):
        """constructor"""
        # call parent constructor
        super().__init__(driver, logger, duration=duration, devices=devices)

    ##################################################
    # Implement application specific _page methods here
    ##################################################



from pages import Page

from selenium.webdriver.common.by import By


class PageSearch(Page):

    def __init__(self, driver, logger):
        super().__init__(driver, logger)

    # Page locators
    linkNroboLink = (By.CSS_SELECTOR, "a[href='/project/nrobo/'] h3 span.package-snippet__name")

    # associated page method
    def nrobo_link_is_present(self) -> bool:
        self.logger.info("Check if nrobo link is available.")
        return self.is_displayed(*self.linkNroboLink)



"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

regex module holds regular expression wrapper methods.

@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import re


def substitute(pattern, repl, string, count=0, flags=0):
    """Return the string obtained by replacing the leftmost
    non-overlapping occurrences of the pattern in string by the
    replacement repl.  repl can be either a string or a callable;
    if a string, backslash escapes in it are processed.  If it is
    a callable, it's passed the Match object and must return
    a replacement string to be used."""
    return re.sub(pattern, repl,string, count=0, flags=flags)



"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

Detection module has methods to detect environment,
nRoBo installation and many more...


@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import os
from pathlib import Path

from nrobo import EnvKeys, Environment
from nrobo import NROBO_PATHS
from nrobo.util.constants import EXT


def developer_machine() -> bool:
    """Return true if Developer machine else False"""
    if (NROBO_PATHS.EXEC_DIR / NROBO_PATHS.PY_PROJECT_TOML_FILE).exists():
        return True

    return False


def production_machine() -> bool:
    """Return true if host machine is in production else false."""
    if os.environ[EnvKeys.ENVIRONMENT] == Environment.PRODUCTION:
        return True

    return False


def a_success(code) -> bool:
    """Return True if code is non-zero (especially in case of command execution) else False"""
    if code:
        return True

    return False


def development_machine() -> bool:
    """Return true if host machine is in a developer machine else false."""
    if os.environ[EnvKeys.ENVIRONMENT] == Environment.DEVELOPMENT:
        return True

    return False


def host_machine_has_nRoBo() -> bool:
    """Returns True if host machine has nRoBo installed already else False"""
    if (NROBO_PATHS.EXEC_DIR / NROBO_PATHS.CONFTEST_PY).exists():
        # if conftest file found on production_machine system, meaning nrobo is already installed there
        return True

    return False


def build_version_from_version_files() -> str:
    """Return build version from version files."""
    from nrobo.util.common import Common
    from nrobo import NROBO_PATHS
    from cli.build import ENV_CLI_SWITCH
    # Grab version number from version yaml files in version/ directory
    return Common.read_yaml(NROBO_PATHS.VERSIONS / f"{ENV_CLI_SWITCH.PROD}{EXT.YAML}", fail_on_failure=False)['version']


def ensure_pathces_dir() -> bool:
    """Ensures that patches dir is present on Host machine"""
    from nrobo import NROBO_PATHS
    import nrobo.cli.detection as detect

    if detect.production_machine() and not detect.developer_machine():
        if not (NROBO_PATHS.NROBO_DIR / NROBO_PATHS.PATCHES).exists():
            os.mkdir(NROBO_PATHS.NROBO_DIR / NROBO_PATHS.PATCHES)



"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================


Definitions of rich formatting styles to be used across nrobo framework
for rich formatted console outputs.

@author: Panchdev Chauhan
@email: erpanchdev@gmail.com
"""
from rich.theme import Theme


class STYLE:
    """Types of styles"""

    TASK = "task"
    STEP = "step"
    HLOrange = "highlight_text_style_orange"
    HLRed = "highlight_text_style_red"
    HLGreen = "highlight_text_style_green"
    WARNING = "warning",
    INFO = "info",
    DANGER = "danger"
    PURPLE4 = "purple4"
    BOLD = "bold"
    ITALIC = "italic"


themes = Theme({  # Defined rich themes
    STYLE.TASK: "bold blue",
    STYLE.STEP: "italic green",
    STYLE.HLOrange: "italic dark_orange3",
    STYLE.HLRed: "italic red",
    STYLE.HLGreen: "italic green",
    STYLE.INFO: "dim cyan",
    STYLE.WARNING: "magenta",
    STYLE.DANGER: "bold red",
    STYLE.BOLD: "bold"
})



"""
@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import pytest
from pages.PagePyPiHome import PagePyPiHome


class TestPyPiHomePage():

    @pytest.mark.unit
    @pytest.mark.skip(reason="Test is not yet complete")
    def test_must_a_failure(self, driver, logger):
        logger.info("This test will fail for sure")
        driver.get("https://google.com")
        raise Exception("Must a fail test")

    @pytest.mark.unit
    def test_this_will_pass_always(self, driver, logger):
        logger.info("this test will pass always")
        driver.get("https://google.com")

    @pytest.mark.unit
    @pytest.mark.skip
    def test_pass_app_name_from_commandline(self, logger, app):
        logger.info(f"Appname={app}")

    @pytest.mark.unit
    @pytest.mark.skip
    def test_pass_url_username_password_from_commandline_example(self, driver, logger, url, username, password):
        logger.info(f"URL={url}, Username={username} and Password={password}")
        driver.get(url)

    @pytest.mark.sanity
    def test_google_branding_displayed_on_home_page(self, driver, logger):
        # Instantiate page object
        page_pypi_home = PagePyPiHome(driver, logger)
        # call page method
        page_pypi_home.open()

        # Asset test condition
        assert page_pypi_home.search_button_present() == True

    @pytest.mark.regression
    def test_nrobo_package_is_available_on_pypi_org(self, driver, logger):
        page_pypi_home = PagePyPiHome(driver, logger)

        page_pypi_home.open()

        page_pypi_home.type_search_keyword("nrobo")

        page_search = page_pypi_home.search()

        # put a checkpoint
        assert page_search.nrobo_link_is_present()

    @pytest.mark.xfail
    @pytest.mark.skip(reason="Wanted to skip it for easy demonstration")
    def test_this_will_fail_for_sure(self, driver, logger):
        raise Exception("Must fail")



import pytest
from selenium.webdriver.common.by import By

from pages import Page


class TestAlertMethods:

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_example_accept_alert(self, driver, logger):
        """Example of accepting alerts"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkJavaScriptAlert = (By.CSS_SELECTOR, "[href='/javascript_alerts']")
        page.click(*lnkJavaScriptAlert)

        btnJSAlert = (By.XPATH, "//button[text()='Click for JS Alert']")
        page.click(*btnJSAlert)
        page.accept_alert()

        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_example_dismiss_alert(self, driver, logger):
        """Example of dismiss alert"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkJavaScriptAlert = (By.CSS_SELECTOR, "[href='/javascript_alerts']")
        page.click(*lnkJavaScriptAlert)

        btnJSAlert = (By.XPATH, "//button[text()='Click for JS Alert']")
        page.click(*btnJSAlert)
        page.dismiss_alert()

        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_send_keys_to_prompt(self, driver, logger):
        """Example of sending keys to the prompt"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkJavaScriptAlert = (By.CSS_SELECTOR, "[href='/javascript_alerts']")
        page.click(*lnkJavaScriptAlert)

        btnJSPrompt = (By.XPATH, "//button[text()='Click for JS Prompt']")
        page.click(*btnJSPrompt)

        page.send_keys_and_accept_alert("Sending keys to the JS Alert prompt.")

        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_example_get_alert_text(self, driver, logger):
        """Example of get alert text"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkJavaScriptAlert = (By.CSS_SELECTOR, "[href='/javascript_alerts']")
        page.click(*lnkJavaScriptAlert)

        btnJSAlert = (By.XPATH, "//button[text()='Click for JS Alert']")
        page.click(*btnJSAlert)
        logger.info(f"Alert text=> {page.get_alert_text()}")
        page.dismiss_alert()

        page.wait_for_a_while(3)




"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

Definitions of nRoBo command line arguments.


@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""

# Define nrobo command line argument parser
import argparse

from nrobo.cli.cli_constants import nCLI, NREPORT
from nrobo.cli.nglobals import Browsers

BOOL_SWITCHES = [
    f"--{nCLI.INSTALL}",
    f"--{nCLI.VERSION}",
    f"--{nCLI.SUPPRESS}",
    "--markers",
    "--exitfirst",
    "--fixtures",
    "--funcargs",
    "--fixtures-per-test",
    "--pdb",
    "--trace",
    "-s",
    "--runxfail",
    "--last-failed",
    "--failed-first",
    "--ff",
    "--nf",
    "--new-first",
    "--cache-clear",
    "--sw",
    "--stepwise",
    "--sw-skip",
    "--stepwise-skip",
    "--verbose",
    "--no-header",
    "--no-summary",
    "--quiet",
    "--disable-warnings",
    "--disable-pytest-warnings",
    "--showlocals",
    "--full-trace",
    "--strict-config",
    "--strict-markers",
    "--strict",
    "--continue-on-collection-errors",
    "--co",
    "--collect-only",
    "--pyargs",
    "--noconftest",
    "--keep-duplicates",
    "--collect-in-virtualenv",
    "--doctest-modules",
    "--doctest-ignore-import-errors",
    "--doctest-continue-on-failure",
    "--version",
    "--trace-config",
    "--debug",
    "--override-ini",
    "--setup-only",
    "--setup-show",
    "--setup-plan",
]

SHOW_ONLY_SWITCHES = [
    "markers",
    "fixtures",
    "funcargs",
    "fixtures-per-test",
    "collect-only",
    "version",
    "setup-plan",
    "co"
]


def nrobo_cli_parser(exit_on_failure=True):
    """Define nRoBo command line arguments

       and return args."""

    parser = argparse.ArgumentParser(
        prog="nrobo",
        description='CLI Switches of nRoBo Test Automation framework', exit_on_error=exit_on_failure)
    parser.add_argument("-i", f"--{nCLI.INSTALL}", help="Install nRoBo requirements and framework on host system",
                        action="store_true")
    parser.add_argument(f"--{nCLI.APP}", help="Name of application under test. Name should not include special chars "
                                              "and should only having alphanumeric values.")
    parser.add_argument(f"--{nCLI.URL}", help="Application url under test.")
    parser.add_argument(f"--{nCLI.USERNAME}", help="Username for login.", default="")
    parser.add_argument(f"--{nCLI.PASSWORD}", help="Password for login.", default="")
    parser.add_argument("-n", f"--{nCLI.INSTANCES}",
                        help="Number of parallel tests to reduce test-run-time. Default value is 1. Meaning single test at a time in sequence.",
                        default=1)
    parser.add_argument(f"--{nCLI.RERUNS}",
                        help=f"Retries to rerun the failed tests n times specified by --{nCLI.RERUNS} switch.",
                        default=0)
    parser.add_argument(f"--{nCLI.RERUNS_DELAY}",
                        help="Delay time in second(s) before a rerun for a failed test. Default is 1 second.",
                        default=1)
    parser.add_argument(f"--{nCLI.REPORT}",
                        help="Defines type of test report. Two types are supported, Simple HTML or Rich Allure report. Options are <html> | <allure>. Default is <html>",
                        default="html")
    parser.add_argument(f"--{nCLI.TARGET}",
                        help="Report name", default=f"{NREPORT.HTML_REPORT_NAME}")
    parser.add_argument(f"--{nCLI.VERSION}",
                        help="Shows nRoBo version", action="store_true")
    parser.add_argument(f"--{nCLI.SUPPRESS}",
                        help="Suppresses upgrade prompt on each test run", action="store_true", default=False)
    parser.add_argument(f"--{nCLI.FULLPAGE_SCREENSHOT}",
                        help="Take full page screenshot", action="store_true", default=False)
    parser.add_argument("-b", f"--{nCLI.BROWSER}", help="""
        Target browser. Default is chrome.
        Options could be:
            {} | {} | {} |
            {} | {} | {} | {}
        """.format(Browsers.CHROME, Browsers.CHROME_HEADLESS, Browsers.ANTI_BOT_CHROME,
                   Browsers.FIREFOX, Browsers.FIREFOX_HEADLESS,
                   Browsers.SAFARI, Browsers.EDGE))
    parser.add_argument(f"--{nCLI.BROWSER_CONFIG}", help="""
            Path of browser-config-file containing additional options that is/are needed to be applied
            before browser instantiation. Each line in file should contain one option only.
            For example: You want to apply, --start-maximized, chrome switch for chrome browser.
            and if the browser-config-file is names as 'chrome_config.txt', then
            the content of file would be as following:
    
            --start-maximized
    
            There will be no conversion taking place by nRoBo! The browser switches will be applied to the browser instance.
            """)
    parser.add_argument("-k", f"--{nCLI.KEY}", help="""
        Only run tests that match the given substring
                            expression. An expression is a python resolvable
                            expression where all names are substring-matched
                            against test names and their parent classes.
    
                            Example: -k 'test_method or test_other' matches all
                            test.yaml functions and classes whose name contains
                            'test_method' or 'test_other', while -k 'not
                            test_method' matches those that don't contain
                            'test_method' in their names. -k 'not test_method
                            and not test_other' will eliminate the matches.
                            Additionally keywords are matched to classes and
                            functions containing extra names in their
                            'extra_keyword_matches' set, as well as functions
                            which have names assigned directly to them. The
                            matching is case-insensitive.
        """)
    parser.add_argument("--alluredir", help="""
        Path to the directory where Allure Pytest will save the test results.
        """)
    parser.add_argument(f"--{nCLI.GRID}", help="""
                Remote Grid server url. Tests will be running on the machine when Grid server is running pointed by Grid url.
                """)
    parser.add_argument("-m", "--marker", help="""
        Only run tests matching given mark expression.
        For example: -m 'mark1 and not mark2'
        Following ready to use markers are registered by nRoBo framework: sanity, regression, ui, api, nogui, and unit.
        
        Usage:
        @pytest.mark.regression
        def test_method1(self, driver, logger):
            ...
            
        @pytest.mark.ui
        def test_method1(self, driver, logger):
            ...
                       
        Then,
        
        To only run regression tests, following switch can be used: 
        nrobo --marker regression OR nrobo -m regression     
        """)
    parser.add_argument("--markers", help="""
        Show markers (builtin, plugin and per-project ones).
        """, action="store_true")
    parser.add_argument("-x", "--exitfirst", help="""
        exit instantly on first error or failed test.yaml.
        """, action="store_true")
    parser.add_argument("--fixtures", help="""
        show available fixtures, sorted by plugin appearance
                            (fixtures with leading '_' are only shown with '-v')
                            """, action="store_true")
    parser.add_argument("--funcargs", help="""
            show available fixtures, sorted by plugin appearance
                                (fixtures with leading '_' are only shown with '-v')
                                """, action="store_true")
    parser.add_argument("--fixtures-per-test", help="show fixtures per test.yaml", action="store_true")
    parser.add_argument("--pdb", help="""
        start the interactive Python debugger on errors or
                            KeyboardInterrupt.
                            """, action="store_true")
    parser.add_argument("--pdbcls", help="""
        --pdbcls=modulename:classname
                            start a custom interactive Python debugger on
                            errors. For example:
                            --pdbcls=IPython.terminal.debugger:TerminalPdb
                            """)
    parser.add_argument("--trace", help="Immediately break when running each test.yaml.", action="store_true")
    parser.add_argument("--capture", help="""
        --capture=method      per-test.yaml capturing method: one of fd|sys|no|tee-sys.
        """)
    parser.add_argument("-s", f"--capture-no", help="shortcut for --capture=no.", action="store_true")
    parser.add_argument("--runxfail", help="""
        report the results of xfail tests as if they were
                            not marked
                            """, action="store_true")
    parser.add_argument("-lf", "--last-failed", help="""
        rerun only the tests that failed at the last run (or
                            all if none failed)
                            """, action="store_true")
    parser.add_argument("--ff", "--failed-first", help="""
        run all tests, but run the last failures first.
                            This may re-order tests and thus lead to repeated
                            fixture setup/teardown.
                            """, action="store_true")
    parser.add_argument("--nf", "--new-first", help="""
        run tests from new files first, then the rest of the
                            tests sorted by file mtime
                            """, action="store_true")
    parser.add_argument("--cache-show", help="""
            --cache-show=[CACHESHOW]
                            show cache contents, don't perform collection or
                            tests. Optional argument: glob (default: '*').
                                """)
    parser.add_argument("--cache-clear", help="""
            remove all cache contents at start of test run.
                                """, action="store_true")
    parser.add_argument("--lfnf", help="""
            --lfnf={all,none}, --last-failed-no-failures={all,none}
                            which tests to run with no previously (known)
                            failures.
                                """)
    parser.add_argument("--sw", "--stepwise", help="""
            exit on test failure and continue from last failing
                            test next time
                                """, action="store_true")
    parser.add_argument("--sw-skip", "--stepwise-skip", help="""
            ignore the first failing test but stop on the next
                            failing test
                                """, action="store_true")
    parser.add_argument("--durations", help="""
            --durations=N         show N slowest setup/test durations (N=0 for all).
                                """)
    parser.add_argument("--durations-min", help="""
            Minimal duration in seconds for inclusion in slowest
                            list. Default 0.005
                                """, default='0.005')
    parser.add_argument("-v", "--verbose", help="""
            increase verbosity.
                                """, action="store_true")
    parser.add_argument("--no-header", help="""
            disable header
                                """, action="store_true")
    parser.add_argument("--no-summary", help="""
            disable summary
            """, action="store_true")
    parser.add_argument("-q", "--quiet", help="""
            decrease verbosity.
                                """, action="store_true")
    parser.add_argument("--verbosity", help="""
            --verbosity=VERBOSE   
            set verbosity. Default is 0.
                                """, default='0')
    parser.add_argument("-r", "--extra-summary", help="""
            -r chars              show extra test summary info as specified by chars:
                            (f)ailed, (E)rror, (s)kipped, (x)failed, (X)passed,
                            (p)assed, (P)assed with output, (a)ll except passed
                            (p/P), or (A)ll. (w)arnings are enabled by default
                            (see --disable-warnings), 'N' can be used to reset
                            the list. (default: 'fE').
                                """)
    parser.add_argument("--disable-warnings", "--disable-pytest-warnings", help="""
                disable warnings summary
                """, action="store_true")
    parser.add_argument("-l", "--showlocals", help="""
                show locals in tracebacks (disabled by default).
                """, action="store_true")
    parser.add_argument("--tb", help="""
                --tb=style.            traceback print mode
                            (auto/long/short/line/native/no).
                """)
    parser.add_argument("--show-capture", help="""
               --show-capture={no,stdout,stderr,log,all}.
                            Controls how captured stdout/stderr/log is shown on
                            failed tests. Default is 'all'.
                """)
    parser.add_argument("--full-trace", help="""
                don't cut any tracebacks (default is to cut).
                """, action="store_true")
    parser.add_argument("--color", help="""
                --color=color. color terminal output (yes/no/auto).
                """)
    parser.add_argument("--code-highlight", help="""
                --code-highlight={yes,no}.
                            Whether code should be highlighted (only if --color
                            is also enabled)
                """)
    parser.add_argument("--pastebin", help="""
                --pastebin=mode.      send failed|all info to bpaste.net pastebin service.
                """)
    parser.add_argument("--junit-xml", help="""
                --junit-xml=path.      create junit-xml style report file at given path.
                """)
    parser.add_argument("--junit-prefix", help="""
                --junit-prefix=str. prepend prefix to classnames in junit-xml output
                """)
    parser.add_argument("-W", "--pythonwarnings", help="""
                -W PYTHONWARNINGS, --pythonwarnings=PYTHONWARNINGS
                            set which warnings to report, see -W option of
                            python itself.
                """)
    parser.add_argument("--maxfail", help="""
                --maxfail=num. exit after first num failures or errors.
                """)
    parser.add_argument("--strict-config", help="""
                any warnings encountered while parsing the `pytest`
                            section of the configuration file raise errors.
                """, action="store_true")
    parser.add_argument("--strict-markers", help="""
                markers not registered in the `markers` section of
                            the configuration file raise errors.
                """, action="store_true")
    parser.add_argument("--strict", help="""
                (deprecated) alias to --strict-markers.
                """, action="store_true")
    parser.add_argument("-c", "--configuration", help="""
                -c file. load configuration from `file` instead of trying to
                            locate one of the implicit configuration files.
                """)
    parser.add_argument("--continue-on-collection-errors", help="""
                --continue-on-collection-errors.
                            Force test execution even if collection errors
                            occur.
                """, action="store_true")
    parser.add_argument("--rootdir", help="""
                --rootdir=ROOTDIR. Define root directory for tests. Can be relative
                            path: 'root_dir', './root_dir',
                            'root_dir/another_dir/'; absolute path:
                            '/home/user/root_dir'; path with variables:
                            '$HOME/root_dir'.
                """)
    parser.add_argument("--co", "--collect-only", help="""
                only collect tests, don't execute them.
                """, action="store_true")
    parser.add_argument("--pyargs", help="""
                try to interpret all arguments as python packages.
                """, action="store_true")
    parser.add_argument("--ignore", help="""
                --ignore=path. ignore path during collection (multi-allowed).
                """)
    parser.add_argument("--ignore-glob", help="""
                --ignore-glob=path. ignore path pattern during collection (multi-
                            allowed).
                """)
    parser.add_argument("--deselect", help="""
                --deselect=nodeid_prefix. deselect item (via node id prefix) during collection
                            (multi-allowed).
                """)
    parser.add_argument("--confcutdir", help="""
                --confcutdir=dir. only load conftest.py's relative to specified dir.
                """)
    parser.add_argument("--noconftest", help="""
                Don't load any conftest.py files.
                """, action="store_true")
    parser.add_argument("--keep-duplicates", help="""
                Keep duplicate tests.
                """, action="store_true")
    parser.add_argument("--collect-in-virtualenv", help="""
                        Don't ignore tests in a local virtualenv directory
                        """, action="store_true")
    parser.add_argument("--import-mode", help="""
                        --import-mode={prepend,append,importlib}. prepend/append to sys.path when importing test
                        """)
    parser.add_argument("--doctest-modules", help="""
                        run doctests in all .py modules
                        """, action="store_true")
    parser.add_argument("--doctest-report", help="""
                        --doctest-report={none,cdiff,ndiff,udiff,only_first_failure}. 
                        choose another output format for diffs on doctest
                            failure
                        """)
    parser.add_argument("--doctest-glob", help="""
                        --doctest-glob=pat. doctests file matching pattern, default: test*.txt
                        """)
    parser.add_argument("--doctest-ignore-import-errors", help="""
                        ignore doctest ImportErrors
                        """, action="store_true")
    parser.add_argument("--doctest-continue-on-failure", help="""
                        for a given doctest, continue to run after the first
                            failure
                        """, action="store_true")
    parser.add_argument("--basetemp", help="""
                        --basetemp=dir. base temporary directory for this test run.(warning:
                            this directory is removed if it exists)
                        """)
    parser.add_argument("-V", "--version", help="""
                        display pytest version and information about
                            plugins.When given twice, also display information
                            about plugins.
                        """, action="store_true")
    # parser.add_argument("-h", "--help", help="""
    #                 show help message and configuration info
    #                 """)
    parser.add_argument("-p", "--plugin-module", help="""
                        -p name               early-load given plugin module name or entry point
                            (multi-allowed).
                            To avoid loading of plugins, use the `no:` prefix,
                            e.g. `no:doctest`.
                        """)
    parser.add_argument("--trace-config", help="""
                        trace considerations of conftest.py files.
                        """, action="store_true")
    parser.add_argument("--debug", help="""
                        store internal tracing debug information in
                            'pytestdebug.log'.
                        """, action="store_true")
    parser.add_argument("-o", "--override-ini", help="""
                        -o OVERRIDE_INI, --override-ini=OVERRIDE_INI
                            override ini option with "option=value" style, e.g.
                            `-o xfail_strict=True -o cache_dir=cache`.
                        """, action="store_true")
    parser.add_argument("--assert", help="""
                        --assert=MODE         Control assertion debugging tools.
                            'plain' performs no assertion debugging.
                            'rewrite' (the default) rewrites assert statements
                            in test modules on import to provide assert
                            expression information.
                        """)
    parser.add_argument("--setup-only", help="""
                        only setup fixtures, do not execute tests.
                        """, action="store_true")
    parser.add_argument("--setup-show", help="""
                        show setup of fixtures while executing tests.
                        """, action="store_true")
    parser.add_argument("--setup-plan", help="""
                        show what fixtures and tests would be executed but
                            don't execute anything.
                        """, action="store_true")
    parser.add_argument("--log-level", help="""
                        --log-level=LEVEL     level of messages to catch/display.
                            Not set by default, so it depends on the root/parent
                            log handler's effective level, where it is "WARNING"
                            by default.
                        """)
    parser.add_argument("--log-format", help="""
                        --log-format=LOG_FORMAT
                            log format as used by the logging module.
                        """)
    parser.add_argument("--log-date-format", help="""
                        --log-date-format=LOG_DATE_FORMAT
                            log date format as used by the logging module.
                        """)
    parser.add_argument("--log-cli-level", help="""
                        --log-cli-level=LOG_CLI_LEVEL
                            cli logging level.
                        """)
    parser.add_argument("--log-cli-format", help="""
                        --log-cli-format=LOG_CLI_FORMAT
                            log format as used by the logging module.
                        """)
    parser.add_argument("--log-cli-date-format", help="""
                        --log-cli-date-format=LOG_CLI_DATE_FORMAT
                            log date format as used by the logging module.
                        """)
    parser.add_argument("--log-file", help="""
                        --log-file=LOG_FILE   path to a file when logging will be written to.
                        """)
    parser.add_argument("--log-file-level", help="""
                        --log-file-level=LOG_FILE_LEVEL
                            log file logging level.
                        """)
    parser.add_argument("--log-file-format", help="""
                        --log-file-format=LOG_FILE_FORMAT
                            log format as used by the logging module.
                        """)
    parser.add_argument("--log-file-date-format", help="""
                        --log-file-date-format=LOG_FILE_DATE_FORMAT
                            log date format as used by the logging module.
                        """)
    parser.add_argument("--log-auto-indent", help="""
                        --log-auto-indent=LOG_AUTO_INDENT
                            Auto-indent multiline messages passed to the logging
                            module. Accepts true|on, false|off or an integer.
                        """)

    return parser.parse_args()
